/*    
    WinDVD Plugin for ATI Remote Wonder
    Copyright 2002-2003 Jonathan Zarate

    This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the
	Free Software Foundation; either version 2 of the License, or (at your
	option) any later version.

    This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
	or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
	for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software Foundation,
	Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "stdafx.h"
#include <stdlib.h>
#include <commctrl.h>
#include "resource.h"
#include "global.h"
#include "actioncodes.h"

#pragma comment(lib, "comctl32")


// for subclassed controls - original wndprocs
WNDPROC OrgListProc;
WNDPROC OrgComboProc;

// INI will be at the same location as DLL
void GetConfigName(char* s) {
	char* p;
	GetModuleFileName(hInst, s, MAX_PATH);
	if ((p = strrchr(s, '\\')) == NULL) p = s; else *(++p) = 0;
	strcat(s, CONFIG_INI);
}

void LoadConfig(void) {
	int i;
	char s[32];
	char t[32];
	char f[MAX_PATH];
	DWORD c;
	CActionCode* a;

	GetConfigName(f);
	for (i = 0; i < MAX_WONDER_KEYS; ++i) {
		ActionMap[i] = 0;
		GetPrivateProfileString("map", ltoa(i, s, 10), "", t, sizeof(t), f);
		c = strtoul(t, NULL, 16);
		// verify the code
		for (a = ActionCodes; a->Name != NULL; ++a) {
			if (c == a->Code) {
				ActionMap[i] = c;
				break;
			}
		}
	}
}

void SaveConfig(void) {
	int i;
	char s[32];
	char t[32];
	char f[MAX_PATH];

	GetConfigName(f);
	for (i = 0; i < MAX_WONDER_KEYS; ++i) {
		WritePrivateProfileString("map", ltoa(i, s, 10), ltoa(ActionMap[i], t, 16), f);
	}
}



LRESULT CALLBACK SubListProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	if ((uMsg == WM_COMMAND) && (GET_WM_COMMAND_ID(wParam, lParam) == 777) &&
        (GET_WM_COMMAND_CMD(wParam, lParam) == CBN_CLOSEUP)) {
		SetFocus(hWnd);	// grab the focus, and the combo will close itself...
		return 0;
	}
	return CallWindowProc(OrgListProc, hWnd, uMsg, wParam, lParam);
}

void SubComboKillFocus(HWND hWnd) {
	char s[256];
	LONG n;
	n = GetWindowLong(hWnd, GWL_USERDATA);
	if (n != -1) {
		GetWindowText(hWnd, s, sizeof(s));
		ListView_SetItemText(GetParent(hWnd), n, 1, s);
	}
	PostMessage(hWnd, WM_CLOSE, 0, 0);
}

LRESULT CALLBACK SubComboProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	switch (uMsg) {
	case WM_KILLFOCUS:
		SubComboKillFocus(hWnd);
		break;
	case WM_KEYDOWN:
		if (wParam == VK_ESCAPE) {
			SetWindowLong(hWnd, GWL_USERDATA, -1);
		}
		break;
	}
	return CallWindowProc(OrgComboProc, hWnd, uMsg, wParam, lParam);
}


void ConfigInitDialog(HWND hWnd) {
	HWND hList;
	RECT r;
	LVCOLUMN lvc;
	LVITEM lvi;
	int n;
	int k;
	char s[256];
	CActionCode* a;

	hList = GetDlgItem(hWnd, IDC_CONFIG_LIST);
	ListView_SetExtendedListViewStyle(hList, 
		ListView_GetExtendedListViewStyle(hList)|LVS_EX_GRIDLINES|LVS_EX_FULLROWSELECT);

	// subclass the list (note: ignore the warnings)
#pragma warning(push)
#pragma warning(disable:4311 4312)
	OrgListProc = (WNDPROC)GetWindowLong(hList, GWL_WNDPROC);
	SetWindowLong(hList, GWL_WNDPROC, (LONG)SubListProc);
#pragma warning(pop)

	GetClientRect(hList, &r);
	n = (r.right - GetSystemMetrics(SM_CXVSCROLL)) / 10;
	lvc.cx = n * 4;
	lvc.mask = LVCF_TEXT|LVCF_WIDTH;
	lvc.pszText = "Remote Wonder Button";
	ListView_InsertColumn(hList, 0, &lvc);
	lvc.cx = n * 6;
	lvc.pszText = "Action";
	ListView_InsertColumn(hList, 1, &lvc);

	for (n = 0; n < MAX_WONDER_KEYS; ++n) {
		s[0] = 0;
		LoadString(hInst, n + IDS_WONDERBUTTONS, s, sizeof(s));
		lvi.iItem = n;
		lvi.iSubItem = 0;
		lvi.pszText = s;
		lvi.mask = LVIF_TEXT;
		k = ListView_InsertItem(hList, &lvi);
		for (a = ActionCodes; a->Name != NULL; ++a) {
            if (a->Code == ActionMap[n]) break;
		}
		ListView_SetItemText(hList, k, 1, a->Name?a->Name:ActionCodes[0].Name);
	}

	ListView_SetItemState(hList, 0, LVIS_FOCUSED|LVIS_SELECTED, LVIS_FOCUSED|LVIS_SELECTED);
}


void ConfigShowActions(HWND hList, int iKey) {
	RECT r;
	HWND hCombo;
	char s[256];
	CActionCode* a;

	SetFocus(hList);

	ListView_EnsureVisible(hList, iKey, FALSE);
	ListView_GetSubItemRect(hList, iKey, 1, LVIR_BOUNDS, &r);
	r.bottom += ((r.bottom - r.top) * 20);

	// create the drop-down list
	hCombo = CreateWindow("COMBOBOX", "", WS_CHILD|WS_VISIBLE|WS_VSCROLL|WS_HSCROLL|CBS_DROPDOWNLIST,
		r.left, r.top, r.right - r.left, r.bottom - r.top, hList, (HMENU)777, hInst, NULL);
	SetWindowLong(hCombo, GWL_USERDATA, iKey);	// pass the current item
	SendMessage(hCombo, WM_SETFONT, SendMessage(hList, WM_GETFONT, 0, 0), FALSE);
	for (a = ActionCodes; a->Name != NULL; ++a) {
		SendMessage(hCombo, CB_ADDSTRING, 0, (LPARAM)a->Name);
	}
	ListView_GetItemText(hList, iKey, 1, s, sizeof(s));
	SendMessage(hCombo, CB_SELECTSTRING, -1, (LPARAM)s);
	SendMessage(hCombo, CB_SHOWDROPDOWN, TRUE, 0);

	// subclass the combo (ignore the warnings)
#pragma warning(push)
#pragma warning(disable:4311 4312)
	OrgComboProc = (WNDPROC)GetWindowLong(hCombo, GWL_WNDPROC);
	SetWindowLong(hCombo, GWL_WNDPROC, (LONG)SubComboProc);
#pragma warning(pop)

	//
	SetFocus(hCombo);
}

void ConfigOK(HWND hWnd) {
	CActionCode* a;
	int n;
	HWND hList;
	char s[256];

	hList = GetDlgItem(hWnd, IDC_CONFIG_LIST);
	for (n = 0; n < MAX_WONDER_KEYS; ++n) {
		ListView_GetItemText(hList, n, 1, s, sizeof(s));
		ActionMap[n] = 0;
		for (a = ActionCodes; a->Name != NULL; ++a) {
			if (strcmp(a->Name, s) == 0) {
				ActionMap[n] = a->Code;
				break;
			}
		}
	}
	SaveConfig();
}

BOOL CALLBACK ConfigDlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	switch (uMsg) {
	case WM_INITDIALOG:
		ConfigInitDialog(hWnd);
		break;
	case WM_COMMAND:
		switch (GET_WM_COMMAND_ID(wParam, lParam)) {
		case IDOK:
			ConfigOK(hWnd);
		case IDCANCEL:
			EndDialog(hWnd, IDOK);
			break;
		}
		break;
	case WM_NOTIFY:
		if (wParam == IDC_CONFIG_LIST) {
			switch (((LPNMHDR)lParam)->code) {
			case NM_CLICK:
				if (((LPNM_LISTVIEW)lParam)->iSubItem == 1) {
					ConfigShowActions(((LPNMHDR)lParam)->hwndFrom, ((LPNM_LISTVIEW)lParam)->iItem);
				}
				break;
			case LVN_KEYDOWN:
				switch (((LPNMLVKEYDOWN)lParam)->wVKey) {
				case VK_F2:
				case VK_F3:
				case VK_SPACE:
					ConfigShowActions(((LPNMHDR)lParam)->hwndFrom,
						ListView_GetNextItem(((LPNMHDR)lParam)->hwndFrom, -1, LVNI_SELECTED));
					break;
				}
				break;
			}
		}
		break;
	default:
		return FALSE;
	}
	return TRUE;
}

void ConfigDialog(HWND hWnd) {
	InitCommonControls();
	DialogBox(hInst, MAKEINTRESOURCE(IDD_CONFIG), hWnd, ConfigDlgProc);
}
